// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stream.h>
#include "FrameOperation.h"
#include "Buffer.h"
#include "Vector.h"

class Concat;

//DECLARE_NODE(Concat)
NODE_INFO(Concat, "Signal:Manip", "INPUT1:INPUT2", "OUTPUT", "INPUT1LENGTH:INPUT2LENGTH:OUTPUTLENGTH")

class Concat : public FrameOperation {
   
   int input1ID;
   int input2ID;
   int input1Length;
   int input2Length;
   int outputLength;

public:
   Concat(string nodeName, ParameterSet params)
   : FrameOperation(nodeName, params)
   {
      input1ID = addInput("INPUT1");
      input2ID = addInput("INPUT2");
      input1Length = dereference_cast<int> (parameters.get("INPUT1LENGTH"));
      input2Length = dereference_cast<int> (parameters.get("INPUT2LENGTH"));
      outputLength = dereference_cast<int> (parameters.get("OUTPUTLENGTH"));

      if (input1Length + input2Length != outputLength)
         cerr << "trouble here!!!\n";
   }

   ~Concat() {}

   virtual void specificInitialize()
   {
      this->FrameOperation::specificInitialize();
   }

   void calculate(int output_id, int count, Buffer &out)
   {
      NodeInput input1 = inputs[input1ID];
      ObjectRef input1Value = input1.node->getOutput(input1.outputID, count);

      Vector<float> &output = object_cast<Vector<float> > (out[count]);
      if (input1Value->status != Object::valid)
      {
         output.status = input1Value->status;
         return;
      }

      NodeInput input2 = inputs[input2ID];
      ObjectRef input2Value = input2.node->getOutput(input2.outputID, count);
      if (input2Value->status != Object::valid)
      {
         output.status = input2Value->status;
         return;
      }

      const Vector<float> &in1 = object_cast<Vector<float> > (input1Value);
      const Vector<float> &in2 = object_cast<Vector<float> > (input2Value);
      
      for (int i=0;i<input1Length;i++)
      {
         output[i]=in1[i];
      }
      for (int i=0;i<input2Length;i++)
      {
         output[i+input1Length]=in2[i];
      }
      
      output.status = Object::valid;
   }

};
